# -*- coding: cp950 -*-
#
# EasyIPConfig
# 
# Copyright (C) 2006 Tsung-Hsiang Chang <vgod@csie.ntu.edu.tw>
# Copyright (C) 2006 Rong-En Fan <rafan@infor.org>
#
# This source file is subject to the new BSD license that is available on 
# http://vgod.tw/projects/EasyIPConfig/License.txt.

from time import ctime

import sys
sys.path.append('.')

import locale
lang = locale.getdefaultlocale()[1]
reload(sys)
sys.setdefaultencoding(lang)

from WindowsRegistry import *
import wx
import os
import re
from wx.lib.masked.ipaddrctrl import IpAddrCtrl

version = '0.9.2.RC1'

class NetSH:
   def __init__(self):
      self.interfaces = {}
      self.getSettings()
      self.__dirty = False

   def getSettings(self):
      if_dump = filter(lambda x: re.compile('^(set|add) (address|dns)').match(x) != None,
                       os.popen('netsh interface dump').readlines())
      for i in if_dump:
         self.parseSetting(i.strip())

   def setDHCPAddress(self, interface):
      self.__dirty = True
      cmd = "netsh interface ip set address \"%s\" dhcp" % (interface)
      print cmd
      os.system(cmd)
      cmd = "netsh interface ip set dns \"%s\" dhcp none" % (interface)
      print cmd
      os.system(cmd)

   def setAddress(self, interface, ip, netmask, gateway, dns):
      self.__dirty = True
      # sanity check
      re_ip = re.compile('^\d+\.\d+.\d+\.\d+$');
      if re_ip.match(ip) != None and \
         re_ip.match(netmask) != None:
         cmd = "netsh interface ip set address \"%s\" static %s %s" \
                   % (interface, ip, netmask ) 
         if re_ip.match(gateway) != None:
            cmd = "%s %s 1" % (cmd, gateway)
         print cmd
         os.system(cmd)

         # dns
         i = 1
         for d in dns:
            if re_ip.match(d) != None:
               if i == 1:
                  cmd = "netsh interface ip set dns \"%s\" static %s PRIMARY" % (interface, d)
               else:
                  cmd = "netsh interface ip add dns \"%s\" %s %d" % (interface, d, i)
               print cmd
               os.system(cmd)
               i = i + 1
      else:
         print "Invalid IP / netmask / gateway"

   def parseSetting(self, line):
      if re.compile('^set address').match(line) != None:
         settings = dict( re.compile('(\w+)+=("[^"]+"|[^ ]+)').findall(line) )
         settings['name'] = settings['name'].strip('"')
         settings['name'] = unicode(settings['name'])
         if not self.interfaces.has_key(settings['name']):
            self.interfaces[settings['name']] = {}
         for k,v in settings.iteritems():
            self.interfaces[settings['name']][k] = v
      elif re.compile('^(set|add) dns').match(line) != None:
         # DNS needs special care
         settings = dict( re.compile('(\w+)+=("[^"]+"|[^ ]+)').findall(line) )
         settings['name'] = settings['name'].strip('"')
         settings['name'] = unicode(settings['name'])
         if not self.interfaces.has_key(settings['name']):
            self.interfaces[settings['name']] = {}
         if settings.has_key('source') and settings['source'] == 'dhcp':
            self.interfaces[settings['name']]['dns'] = 'dhcp'
         else:
            # parse dns server lists
            if not self.interfaces[settings['name']].has_key('dns'):
                self.interfaces[settings['name']]['dns'] = []
            self.interfaces[settings['name']]['dns'].append(settings['addr'])

   
   def getInterfaces(self):
      return self.interfaces.keys()
   
   def getConfig(self, interface):
      if self.__dirty:
         self.__init__()
      print self.interfaces.keys()
      return self.interfaces[interface]

class MainWin(wx.Frame):
   def __init__(self):
      title = "Easy IP Config %s - by vgod" % (version)
      wx.Frame.__init__(self, None, title=title, size=(575,345))
      self.SetIcon(wx.Icon('ip-cfg.ico', wx.BITMAP_TYPE_ICO))
      self.panel = MyPanel(self)
      self.Show(True)

class MyPanel(wx.Panel):
   def __init__(self, parent):
      wx.Panel.__init__(self, parent)
      self.netsh = NetSH()
      self.reg = WindowsRegistry(project="Easy IP Config", write=1)
      self.__createControls()
      self.__loadSettings()
      self.__loadHistory()

   def __loadHistory(self):
      history = {}
      self.cf_history.Clear()
      ifs = self.getCurrentInterface()
      try:
         history = self.reg.pget(ifs)
         #print history
         for ip, data in history.iteritems():
            self.cf_history.Append(ip, data)
         self.cf_history.SetSelection(0)
      except:
         print "Can't load history(%s)" % ifs

   def __loadSettings(self):
      i = self.getCurrentInterface()
      print "Load interface " + i
      if i == '':
         return
      conf = self.netsh.getConfig(i) 
      print conf
      self.__clearTxtCtrls()
      if conf['source'] == 'dhcp':
         self.__enableTxtCtrls(False)
         self.rbDHCP.SetValue(1)
      else:
         self.__enableTxtCtrls(True)
         self.rbStaticIP.SetValue(1)
         self.txtIP.SetValue(conf['addr'])
         self.txtNetmask.SetValue(conf['mask'])
         if conf.has_key('gateway'): self.txtGateway.SetValue(conf['gateway'])
         if conf.has_key('dns'):
            i = 0
            for d in conf['dns']:
               self.txtDNS[i].SetValue(d)
               i = i + 1


   def __enableTxtCtrls(self, enable):
      self.__enableCtrls(enable, self.txtIP, self.txtNetmask, self.txtGateway)
      self.__enableCtrls(enable, self.txtDNS)

   def __clearTxtCtrls(self):
      self.__clearCtrls(self.txtIP, self.txtNetmask, self.txtGateway)
      self.__clearCtrls(self.txtDNS)

   def __enableCtrls(self, enable, *ctrls):
      for c in ctrls:
         if type(c) == list:
            for m in c:
               self.__enableCtrls(enable, m)
         else:
            if enable:
               c.Enable()
            else:
               c.Disable()

   def __clearCtrls(self, *ctrls):
      for c in ctrls:
         if type(c) == list:
            for m in c:
               self.__clearCtrls(m)
         else:
            c.Clear()


   def getCurrentInterface(self):
      return self.selInterface.GetValue()

   def __createControls(self):
      sizer = wx.BoxSizer(wx.VERTICAL)
      sizer.Add(self.__createListInterfaces(), 0, wx.ALIGN_CENTER | wx.ALL, 10)
      sizer.Add(self.__createConfigControls(), 0, wx.ALIGN_CENTER | wx.ALL, 25)
      sizer.Add(self.__createBtnOK(), 0, wx.ALIGN_CENTER | wx.ALL, 10)
      self.SetSizerAndFit(sizer)

   def __createConfigControls(self):
      sizer = wx.BoxSizer(wx.VERTICAL)
      # IP
      self.rbDHCP = wx.RadioButton(self, -1, label="DHCP") 
      self.rbDHCP.SetValue(1)
      self.rbStaticIP = wx.RadioButton(self, -1, label="Static IP") 
      sizer.Add( self.rbDHCP, 0, wx.TOP, 5 )
      sizer.Add( self.rbStaticIP, 0, wx.TOP, 5)
      sizer.Add(self.__createTextIP(), 0, wx.TOP, 2)
      self.rbDHCP.Bind(wx.EVT_RADIOBUTTON, self.OnAddrTypeChanged)
      self.rbStaticIP.Bind(wx.EVT_RADIOBUTTON, self.OnAddrTypeChanged)
      # DNS
      sizer.Add(self.__createTextDNS(), 0, wx.TOP, 2)
      return sizer

   def OnAddrTypeChanged(self, evt):
      if evt.GetEventObject() is self.rbDHCP:
         self.__enableTxtCtrls(False)
      else:
         self.__enableTxtCtrls(True)

   def __createListInterfaces(self):
      sizer = wx.BoxSizer(wx.HORIZONTAL)
      interfaces = self.netsh.getInterfaces()
#      ifs = os.popen('netsh interface show interface').readlines()[3:-1]
#      interfaces = map( lambda x: re.split('[ \n]+',x)[2], ifs)
      combo = wx.ComboBox(self, choices=interfaces, style=wx.CB_READONLY)
      combo.Bind(wx.EVT_COMBOBOX, self.OnSelInterfaceChanged)
      combo.SetSelection(0)
      self.selInterface = combo
      sizer.Add(wx.StaticText(self, -1, "Interfaces: "), 0, wx.ALIGN_CENTER_VERTICAL)
      sizer.Add(combo)
      sizer.AddSpacer(wx.Size(80,10))
      sizer.Add(self.__createConfHistory(), 1, wx.ALIGN_RIGHT | wx.ALIGN_TOP)
      # how to align to right?
      return sizer

   def __createConfHistory(self):
      sizer = wx.BoxSizer(wx.HORIZONTAL)
      self.cf_history = wx.ComboBox(self, style=wx.CB_READONLY)
      self.cf_history.Bind( wx.EVT_COMBOBOX, self.OnSelHistoryChanged)
      sizer.Add(wx.StaticText(self, -1, "Profiles: "), 0, wx.ALIGN_CENTER_VERTICAL)
      sizer.Add(self.cf_history)
      return sizer

   def OnSelHistoryChanged(self, evt):
      i = self.cf_history.GetSelection()
      data = self.cf_history.GetClientData(i)
      self.txtIP.SetValue(data['ip'])
      self.txtNetmask.SetValue(data['netmask'])
      self.txtGateway.SetValue(data['gateway'])
      i = 0
      for d in data['dns']:
         self.txtDNS[i].SetValue(d)
         i = i + 1


   def __createTextIP(self):
      sizer = wx.GridSizer(3, 2)
      self.txtIP = self.__createTextAndIPCtrl(sizer, "IP: ")
      self.txtNetmask = self.__createTextAndIPCtrl(sizer, "Netmask: ")
      self.txtGateway = self.__createTextAndIPCtrl(sizer, "Gateway: ")
      return sizer

   def __createTextDNS(self):
      sizer = wx.GridSizer(3, 2)
      self.txtDNS = []
      self.txtDNS.append(self.__createTextAndIPCtrl(sizer, "DNS1: "))
      self.txtDNS.append(self.__createTextAndIPCtrl(sizer, "DNS2: "))
      return sizer

   def __createTextAndIPCtrl(self, sizer, text):
      sizer.Add(wx.StaticText(self, -1, text), 0, wx.ALIGN_CENTER_VERTICAL)
      ctrl = IpAddrCtrl(self, -1)
      sizer.Add(ctrl, 0, wx.ALIGN_CENTER_VERTICAL)
      return ctrl


   def __createBtnOK(self):
      btn = wx.Button(self, wx.ID_ANY, 'OK')
      btn.Bind(wx.EVT_BUTTON, self.OnClickOK)
      return btn

   def OnSelInterfaceChanged(self, event):
      self.__loadSettings()
      self.__loadHistory()

   def OnClickOK(self,event):
      ifs = self.getCurrentInterface()
      if self.rbDHCP.GetValue():
         self.netsh.setDHCPAddress(ifs)
      else:
         ip = self.txtIP.GetAddress()
         netmask = self.txtNetmask.GetAddress()
         gateway = self.txtGateway.GetAddress()
         dns = []
         for txt in self.txtDNS:
            dns.append(txt.GetAddress())
         print ifs + " " + ip
         self.netsh.setAddress(ifs, ip, netmask, gateway, dns)
         self.saveSettings(ifs, ip, netmask, gateway, dns)

   def saveSettings(self, ifs, ip, netmask, gateway, dns):
      try:
         history = self.reg.pget(ifs)
      except:
         print "init %s history" % ifs
         self.reg.pset(ifs, {})
         history = {}
      history[ip] = {'ip':ip, 'netmask':netmask, 'gateway':gateway, 'dns':dns}
      self.reg.pset(ifs, history)
      self.cf_history.Append(ip, history[ip])
      self.cf_history.SetSelection(self.cf_history.GetCount()-1)
      self.__loadSettings()

log = open("easy-ip-cfg.log", "a")
log.write("%s\n" % ctime())
sys.stdout = log
sys.stderr = log

os.putenv('COMSPEC', 'cmd.exe')
app = wx.PySimpleApp()
MainWin()
app.MainLoop()

log.write("\n")
